CallSafety

Enum CallSafety 

pub enum CallSafety {
    Inherit,
    Safe,
    Unsafe,
}
Whether we statically know something about the safety of this call, that takes precedence over the safety of the callee’s signature.

Variants



Inherit

As safe as the callee’s signature.



Safe

Safe despite an unsafe signature: desugared drops, and calls to safe #[target_feature] functions from contexts that enable the required features.



Unsafe

Unsafe despite a safe signature: calls to #[target_feature] functions from contexts that don’t enable the required features, and explicit calls to Drop methods.

Implementations

Source

impl CallSafety

pub fn is_inherit(&self) -> bool

pub fn is_safe(&self) -> bool

pub fn is_unsafe(&self) -> bool