SwitchData

Struct SwitchData 

pub struct SwitchData {
    pub scrutinee: SwitchScrutinee,
    pub branches: Vec<(ConstantExpr, BranchId)>,
    pub fallback: Option<BranchId>,
}
A branching operation.

Fields

scrutinee: SwitchScrutinee

The value to branch over.

branches: Vec<(ConstantExpr, BranchId)>

Which branch to take for each value of the scrutinee. Several values may point to the same branch, and not all values may be accounted for.

If the scrutinee is an operand, the constant expressions are literal values. If the scrutinee is a discriminant read, the expressions are of the form ConstantExprKind::Discriminant.

fallback: Option<BranchId>

Branch to use if the scrutinee didn’t match any of the values above. None if the set of branch values is known to be exhaustive.

Implementations

Source

impl SwitchData

pub fn as_if(&self) -> Option<(BranchId, BranchId)>

If this is a switch over a boolean, return its then and else branches.

pub fn group_by_branch(&self) -> IndexVec<BranchId, Vec<ConstantExpr>>

Group the explicit switch values by the branch they select.