pub enum Body {
Unstructured(ExprBody),
Structured(ExprBody),
TargetDispatch(SeqHashMap<TargetTriple, FunDeclRef>),
Extern(String),
Intrinsic {
name: String,
arg_names: Vec<Option<String>>,
},
Opaque,
Missing,
Error(Error),
}Expand description
The body of a function.
A normal function has a body that’s either structured or unstructured. These are two equivalent
representations of the same function body, which only differ in how control-flow is
represented. By default bodies are structured; passing --ullbc to Charon makes bodies
unstructured.
Besides these, some functions have virtual bodies or no body, see the doc for each variant.
Variants
Unstructured(ExprBody)
Body represented as a control-flow graph (CFG), i.e. with numbered blocks and jumps/GOTOs
between them. This is available when passing --ullbc to Charon.
This is also the same structure as rustc’s MIR, and is in fact a direct translation of it.
Structured(ExprBody)
Body represented with structured control flow, i.e. with nested if/match/loop blocks.
This is available when --ullbc is not passed to Charon.
This structure is recovered from the unstructured body in the ullbc_to_llbc pass.
TargetDispatch(SeqHashMap<TargetTriple, FunDeclRef>)
A façade body that dispatches to one of several per-target function bodies. This is created
during multi-target merging (using --target) to represent functions that have a different
implementation across targets.
Extern(String)
Function declared in an extern { ... } block. The string is the foreign symbol name.
Intrinsic
Rust intrinsic function. This has no body and describes a “built-in” operation that must be handled by the codegen backend.
Fields
Opaque
A body that the user chose not to translate, based on opacity settings like
--include/--opaque.
Missing
A body that was not available.
These can occur when using a sysroot that doesn’t have MIR for all the standard library
functions. This is the case of the sysroot that ships with every Rust toolchain, but Charon
uses a custom-built sysroot that does. So this shouldn’t happen unless you’re passing
--sysroot to charon.
Error(Error)
We encountered an error while translating this body.
Implementations
Sourceimpl Body
impl Body
pub fn is_unstructured(&self) -> bool
pub fn is_structured(&self) -> bool
pub fn is_target_dispatch(&self) -> bool
pub fn is_extern(&self) -> bool
pub fn is_intrinsic(&self) -> bool
pub fn is_opaque(&self) -> bool
pub fn is_missing(&self) -> bool
pub fn is_error(&self) -> bool
Sourceimpl Body
impl Body
pub fn as_unstructured(&self) -> Option<&ExprBody>
pub fn as_structured(&self) -> Option<&ExprBody>
pub fn as_target_dispatch( &self, ) -> Option<&SeqHashMap<TargetTriple, FunDeclRef>>
pub fn as_extern(&self) -> Option<&String>
pub fn as_intrinsic(&self) -> Option<(&String, &Vec<Option<String>>)>
pub fn as_opaque(&self) -> Option<()>
pub fn as_missing(&self) -> Option<()>
pub fn as_error(&self) -> Option<&Error>
Sourceimpl Body
impl Body
pub fn as_unstructured_mut(&mut self) -> Option<&mut ExprBody>
pub fn as_structured_mut(&mut self) -> Option<&mut ExprBody>
pub fn as_target_dispatch_mut( &mut self, ) -> Option<&mut SeqHashMap<TargetTriple, FunDeclRef>>
pub fn as_extern_mut(&mut self) -> Option<&mut String>
pub fn as_intrinsic_mut( &mut self, ) -> Option<(&mut String, &mut Vec<Option<String>>)>
pub fn as_opaque_mut(&mut self) -> Option<()>
pub fn as_missing_mut(&mut self) -> Option<()>
pub fn as_error_mut(&mut self) -> Option<&mut Error>
Sourceimpl Body
impl Body
pub fn to_unstructured(self) -> Option<ExprBody>
pub fn to_structured(self) -> Option<ExprBody>
pub fn to_target_dispatch(self) -> Option<SeqHashMap<TargetTriple, FunDeclRef>>
pub fn to_extern(self) -> Option<String>
pub fn to_intrinsic(self) -> Option<(String, Vec<Option<String>>)>
pub fn to_opaque(self) -> Option<()>
pub fn to_missing(self) -> Option<()>
pub fn to_error(self) -> Option<Error>